@@ -485,7 +485,7 @@ PL_v_special_s = enclose('|'.join (
 
 PL_sb_postfix_adj = {
     'general' : ['(?!major|lieutenant|brigadier|adjutant)\S+'],
-    'martial' : "court".split(),
+    'martial' : ['court'],
 }
 
 for k in PL_sb_postfix_adj.keys():
@@ -495,8 +495,7 @@ for k in PL_sb_postfix_adj.keys():
 
 PL_sb_postfix_adj_stems = '(' + '|'.join(PL_sb_postfix_adj.values()) + ')(.*)'
 
-# PL_sb_military = 'major|lieutenant|brigadier|adjutant|quartermaster'
-# PL_sb_general = '((?!'+PL_sb_military+r').*?)((-|\s+)general)'
+
 
 PL_prep = enclose('|'.join( """
     about above across after among around at athwart before behind
@@ -554,8 +553,6 @@ PL_v_irregular_pres = {
 "do"    : "do",    "do"    : "do",    "does"   : "do",
 }
 
-PL_v_irregular_pres_keys = enclose('|'.join(PL_v_irregular_pres.keys()))
-
 PL_v_ambiguous_pres = {
 #   1st PERS. SING.     2ND PERS. SING.     3RD PERS. SINGULAR
 #               3RD PERS. (INDET.)  
@@ -582,11 +579,11 @@ PL_v_ambiguous_pres = {
 PL_v_ambiguous_pres_keys = enclose('|'.join(PL_v_ambiguous_pres.keys()));
 
 
-PL_v_irregular_non_pres = enclose('|'.join ((
+PL_v_irregular_non_pres = (
 "did", "had", "ate", "made", "put", 
 "spent", "fought", "sank", "gave", "sought",
 "shall", "could", "ought", "should",
-)))
+)
 
 PL_v_ambiguous_non_pres = enclose('|'.join ((
 "thought", "saw", "bent", "will", "might", "cut",
@@ -594,11 +591,10 @@ PL_v_ambiguous_non_pres = enclose('|'.join ((
 
 # "..oes" -> "..oe" (the rest are "..oes" -> "o")
 
-PL_v_oes_oe = enclose('|'.join ( """
-    .*shoes  .*hoes  .*toes
-    canoes   floes   oboes  roes  throes  woes
-    """.split()
-))
+PL_v_oes_oe = ('canoes', 'floes', 'oboes', 'roes', 'throes', 'woes')
+PL_v_oes_oe_endings_size4 = ('hoes', 'toes')
+PL_v_oes_oe_endings_size5 = ('shoes')
+
 
 PL_count_zero = (
 "0", "no", "zero", "nil"
@@ -728,14 +724,15 @@ no_classical = dict((k,0) for k in def_classical.keys())
 
 #TODO: .inflectrc file does not work
 # can't just execute methods from another file like this
-for rcfile in (pathjoin(dirname(__file__), '.inflectrc'),
-               expanduser(pathjoin(('~'), '.inflectrc'))):
-    if isfile(rcfile):
-        try:
-            execfile(rcfile)
-        except:
-            print3("\nBad .inflectrc file (%s):\n" % rcfile)
-            raise BadRcFileError
+
+#for rcfile in (pathjoin(dirname(__file__), '.inflectrc'),
+#               expanduser(pathjoin(('~'), '.inflectrc'))):
+#    if isfile(rcfile):
+#        try:
+#            execfile(rcfile)
+#        except:
+#            print3("\nBad .inflectrc file (%s):\n" % rcfile)
+#            raise BadRcFileError
 
 
 
@@ -752,10 +749,9 @@ class engine:
         self.PL_adj_user_defined  = []
         self.A_a_user_defined   = []
 
-    #TODO checkpat is not the correct routine for checking the plural (i.e. replace) strings
     def def_noun(self, singular, plural):
         self.checkpat(singular)
-        #checkpat(plural)
+        self.checkpatplural(plural)
         self.PL_sb_user_defined.extend((singular, plural))
         return 1
 
@@ -766,15 +762,15 @@ class engine:
         self.checkpat(s1)
         self.checkpat(s2)
         self.checkpat(s3)
-        #checkpat(p1)
-        #checkpat(p2)
-        #checkpat(p3)
+        self.checkpatplural(p1)
+        self.checkpatplural(p2)
+        self.checkpatplural(p3)
         self.PL_v_user_defined.extend((s1, p1, s2, p2, s3, p3))
         return 1
 
     def def_adj(self, singular, plural):
         self.checkpat(singular)
-        #checkpat(plural)
+        self.checkpatplural(plural)
         self.PL_adj_user_defined.extend((singular, plural))
         return 1
 
@@ -799,6 +795,20 @@ class engine:
         except reerror:
             print3("\nBad user-defined singular pattern:\n\t%s\n" % pattern)
             raise BadUserDefinedPatternError
+
+    def checkpatplural(self, pattern):
+        '''
+        check for errors in a regex replace pattern
+        '''
+        return
+        # can't find a pattern that doesn't pass the following test:
+        #if pattern is None:
+        #    return
+        #try:
+        #    resub('', pattern, '')
+        #except reerror:
+        #    print3("\nBad user-defined plural pattern:\n\t%s\n" % pattern)
+        #    raise BadUserDefinedPatternError
        
     def ud_match(self, word, wordlist):
         for i in range(len(wordlist)-2, -2, -2): # backwards through even elements
@@ -897,6 +907,8 @@ class engine:
         A but take a matchobject
         use groups 1 and 3 in matchobject
         '''
+        if matchobject.group(3) is None:
+            return self.A(matchobject.group(1))            
         return self.A(matchobject.group(1), matchobject.group(3))
 
     def NOmo(self, matchobject):
@@ -990,10 +1002,13 @@ class engine:
         """
         if '|' in inflected:
             inflected = inflected.split('|')[self.classical_dict['all']]
-        if orig == "I": return inflected
-        if orig == orig.upper(): return inflected.upper()
-        if orig[0] == orig[0].upper(): return '%s%s' % (inflected[0].upper(),
-                                                      inflected[1:])
+        if orig == "I":
+            return inflected
+        if orig == orig.upper():
+            return inflected.upper()
+        if orig[0] == orig[0].upper():
+            return '%s%s' % (inflected[0].upper(),
+                             inflected[1:])
         return inflected
 
     def partition_word(self, mystr):
@@ -1005,7 +1020,8 @@ class engine:
 
     def PL(self, mystr, count=None):
         pre, word, post = self.partition_word(mystr)
-        if not word: return mystr
+        if not word:
+            return mystr
         plural = self.postprocess(word,
               self._PL_special_adjective(word, count)
               or self._PL_special_verb(word, count)
@@ -1014,20 +1030,23 @@ class engine:
 
     def PL_N(self, mystr, count=None):
         pre, word, post = self.partition_word(mystr)
-        if not word: return mystr
+        if not word:
+            return mystr
         plural = self.postprocess(word, self._PL_noun(word, count))
         return "%s%s%s" % (pre, plural, post)
 
     def PL_V(self, mystr, count=None):
         pre, word, post = self.partition_word(mystr)
-        if not word: return mystr
+        if not word:
+            return mystr
         plural = self.postprocess(word, self._PL_special_verb(word, count)
               or self._PL_general_verb(word, count))
         return "%s%s%s" % (pre, plural, post)
 
     def PL_ADJ(self, mystr, count=None):
         pre, word, post = self.partition_word(mystr)
-        if not word: return mystr
+        if not word:
+            return mystr
         plural = self.postprocess(word, self._PL_special_adjective(word, count)
               or word)
         return "%s%s%s" % (pre, plural, post)
@@ -1050,19 +1069,27 @@ class engine:
     def _PL_eq(self, word1, word2, PL):
         classval = self.classical_dict.copy()
         self.classical_dict = all_classical.copy()
-        if word1 == word2: return "eq"
-        if word1 == PL(word2): return "p:s"
-        if PL(word1) == word2: return "s:p"
+        if word1 == word2:
+            return "eq"
+        if word1 == PL(word2):
+            return "p:s"
+        if PL(word1) == word2:
+            return "s:p"
         self.classical_dict = no_classical.copy()
-        if word1 == PL(word2): return "p:s"
-        if PL(word1) == word2: return "s:p"
+        if word1 == PL(word2):
+            return "p:s"
+        if PL(word1) == word2:
+            return "s:p"
         self.classical_dict = classval.copy()
 
         if PL == self.PL or PL == self.PL_N:
-            if self._PL_check_plurals_N(word1, word2): return "p:p"
-            if self._PL_check_plurals_N(word2, word1): return "p:p"
+            if self._PL_check_plurals_N(word1, word2):
+                return "p:p"
+            if self._PL_check_plurals_N(word2, word1):
+                return "p:p"
         if PL == self.PL or PL == self.PL_ADJ:
-            if self._PL_check_plurals_ADJ(word1, word2): return "p:p"
+            if self._PL_check_plurals_ADJ(word1, word2):
+                return "p:p"
         return False
 
     def _PL_reg_plurals(self, pair, stems, end1, end2):
@@ -1080,8 +1107,10 @@ class engine:
 
     def _PL_check_plurals_N(self, word1, word2):
         pair = "%s|%s" % (word1, word2)
-        if pair in PL_sb_irregular_s.values(): return True
-        if pair in PL_sb_irregular.values(): return True
+        if pair in PL_sb_irregular_s.values():
+            return True
+        if pair in PL_sb_irregular.values():
+            return True
 
         for (stems, end1, end2) in (
                    (PL_sb_C_a_ata_stems,   "as","ata"),
@@ -1107,9 +1136,9 @@ class engine:
 
     def _PL_check_plurals_ADJ(self, word1, word2):
 #VERSION: tuple in endswith requires python 2.5
-#TODO PL_ADJ_eq shouldn't return true from dogmata' . Check that it doesn't
         word1a = word1[:word1.rfind("'")] if word1.endswith(("'s","'")) else ''
         word2a = word2[:word2.rfind("'")] if word2.endswith(("'s","'")) else ''
+        #TODO: BUG? report upstream. I don't think you should chop off the s'
         word1b = word1[:-2] if word1.endswith("s'") else ''
         word2b = word2[:-2] if word2.endswith("s'") else ''
 
@@ -1128,7 +1157,7 @@ class engine:
             if word2b and ( self._PL_check_plurals_N(word1b, word2b)
                             or self._PL_check_plurals_N(word2b, word1b) ):
                 return True
-
+        
         return False
 
     def get_count(self, count=None):
@@ -1148,19 +1177,23 @@ class engine:
 
 # DEFAULT TO PLURAL
 
-        if count==1: return word
+        if count==1:
+            return word
 
 # HANDLE USER-DEFINED NOUNS
 
         value = self.ud_match(word, self.PL_sb_user_defined)
-        if value is not None: return value
+        if value is not None:
+            return value
 
 
 # HANDLE EMPTY WORD, SINGULAR COUNT AND UNINFLECTED PLURALS
 
-        if word == '': return word
+        if word == '':
+            return word
 
-        if search(r"^%s$" % PL_sb_uninflected, word, IGNORECASE): return word
+        if search(r"^%s$" % PL_sb_uninflected, word, IGNORECASE):
+            return word
 
         if (self.classical_dict['herd'] and
                search(r"^%s$" % PL_sb_uninflected_herd, word, IGNORECASE)):
@@ -1183,31 +1216,38 @@ class engine:
                 return "%s%s" % (self._PL_noun(mo.group(1), 2), mo.group(2))
 
 # HANDLE PRONOUNS
-# BUG does not keep case: "about ME" -> "about us". bug not fixed here
+
+# TODO: BUG does not keep case: "about ME" -> "about us". bug not fixed here
         mo = search(r"^((?:%s)\s+)(%s)$" % (PL_prep, PL_pron_acc_keys), word,
                                                          IGNORECASE)
-        if mo: return "%s%s" % (mo.group(1), PL_pron_acc[mo.group(2).lower()])
-     
+        if mo:
+            return "%s%s" % (mo.group(1), PL_pron_acc[mo.group(2).lower()])
+
         try:
             return PL_pron_nom[word.lower()]
-        except KeyError: pass
+        except KeyError:
+            pass
 
         try:
             return PL_pron_acc[word.lower()]
-        except KeyError: pass
+        except KeyError:
+            pass
 
 # HANDLE ISOLATED IRREGULAR PLURALS 
 
         mo = search(r"(.*)\b(%s)$" % PL_sb_irregular_keys, word, IGNORECASE)
-        if mo: return "%s%s" % (mo.group(1),
-                                PL_sb_irregular.get(mo.group(2),
-                                    PL_sb_irregular[mo.group(2).lower()]))
+        if mo:
+            return "%s%s" % (mo.group(1),
+                             PL_sb_irregular.get(mo.group(2),
+                             PL_sb_irregular[mo.group(2).lower()]))
 
         mo = search(r"(%s)$" % PL_sb_U_man_mans, word, IGNORECASE)
-        if mo: return "%ss" % mo.group(1)
+        if mo:
+            return "%ss" % mo.group(1)
 
         mo = search(r"(\S*)quy$", word, IGNORECASE)
-        if mo: return "%squies" % mo.group(1)
+        if mo:
+            return "%squies" % mo.group(1)
 
         mo = search(r"(\S*)(person)$", word, IGNORECASE)
         if mo:
@@ -1226,12 +1266,14 @@ class engine:
                   (r"(.*)foot$", "%sfeet"),
                  ):
             mo = search(a[0], word, IGNORECASE)
-            if mo: return a[1] % mo.group(1)
+            if mo:
+                return a[1] % mo.group(1)
 
 
 # HANDLE UNASSIMILATED IMPORTS
 
-        if search(r"(.*)ceps$", word, IGNORECASE): return word
+        if search(r"(.*)ceps$", word, IGNORECASE):
+            return word
 
 
         for a in (
@@ -1246,7 +1288,8 @@ class engine:
                   (r"(%s)$" % PL_sb_U_a_ae, "%se"),
                  ):
             mo = search(a[0], word, IGNORECASE)
-            if mo: return a[1] % mo.group(1)
+            if mo:
+                return a[1] % mo.group(1)
 
 
 # HANDLE INCOMPLETELY ASSIMILATED IMPORTS
@@ -1273,33 +1316,38 @@ class engine:
                   (r"(%s)$" % PL_sb_C_i, "%si"),
                  ):
                 mo = search(a[0], word, IGNORECASE)
-                if mo: return a[1] % mo.group(1)
+                if mo:
+                    return a[1] % mo.group(1)
 
 
 # HANDLE SINGULAR NOUNS ENDING IN ...s OR OTHER SILIBANTS
         mo = search(r"(%s)$" % PL_sb_singular_s, word, IGNORECASE)
-        if mo: return "%ses" % mo.group(1)
+        if mo:
+            return "%ses" % mo.group(1)
 
 # TODO: not sure if this makes a difference. Wouldn't special words
 # ending with 's' always have been caught, regardless of them starting
-# with a capitla letter (i.e. bieing names)
+# with a capital letter (i.e. being names)
 # It makes sense below to do this for words ending in 'y' so that
 # Sally -> Sallys. But not sure it makes sense here. Where is the case
 # of a word ending in s that is caught here and would otherwise have been
 # caught below?
         if (self.classical_dict['names']):
             mo = search(r"([A-Z].*s)$", word)
-            if mo: return "%ses" % mo.group(1)
+            if mo:
+                return "%ses" % mo.group(1)
 
         mo = search(r"^(.*[^z])(z)$", word, IGNORECASE)
-        if mo: return "%szzes" % mo.group(1)
+        if mo:
+            return "%szzes" % mo.group(1)
 
         for a in (
                   (r"^(.*)([cs]h|x|zz|ss)$",  "%s%ses"),
 #                  (r"(.*)(us)$", "%s%ses"),
                  ):
             mo = search(a[0], word, IGNORECASE)
-            if mo: return a[1] % (mo.group(1), mo.group(2))
+            if mo:
+                return a[1] % (mo.group(1), mo.group(2))
 
 
 # HANDLE ...f -> ...ves
@@ -1311,19 +1359,23 @@ class engine:
                   (r"(.*)arf$", "%sarves"),
                  ):
             mo = search(a[0], word, IGNORECASE)
-            if mo: return a[1] % mo.group(1)
+            if mo:
+                return a[1] % mo.group(1)
 
 # HANDLE ...y
 
         mo = search(r"(.*[aeiou])y$", word, IGNORECASE)
-        if mo: return "%sys" % mo.group(1)
+        if mo:
+            return "%sys" % mo.group(1)
 
         if (self.classical_dict['names']):
             mo = search(r"([A-Z].*y)$", word)
-            if mo: return "%ss" % mo.group(1)
+            if mo:
+                return "%ss" % mo.group(1)
 
         mo = search(r"(.*)y$", word, IGNORECASE)
-        if mo: return "%sies" % mo.group(1)
+        if mo:
+            return "%sies" % mo.group(1)
 
 
 # HANDLE ...o
@@ -1334,7 +1386,8 @@ class engine:
                   (r"o$", "es"),
                  ):
             mo = search(a[0], word, IGNORECASE)
-            if mo: return "%s%s" % (word, a[1])
+            if mo:
+                return "%s%s" % (word, a[1])
 
 # OTHERWISE JUST ADD ...s
 
@@ -1344,67 +1397,77 @@ class engine:
     def _PL_special_verb(self, word, count=None):
         count = self.get_count(count)
 # TODO: should zero make count 2 not return here: check Perl code
-        if str(count).lower() in PL_count_zero: return False
+        if str(count).lower() in PL_count_zero:
+            return False
 
-        if count==1: return word
+        if count==1:
+            return word
 
 # HANDLE USER-DEFINED VERBS
 
         value = self.ud_match(word, self.PL_v_user_defined)
-        if value is not None: return value
+        if value is not None:
+            return value
 
 # HANDLE IRREGULAR PRESENT TENSE (SIMPLE AND COMPOUND)
 
-        mo = search(r"^(%s)((\s.*)?)$" % PL_v_irregular_pres_keys,
-                    word, IGNORECASE)
-        if mo: return "%s%s" % (PL_v_irregular_pres[mo.group(1).lower()],
-                                 mo.group(2))
+        lowerword = word.lower()
+        try:
+            firstword = lowerword.split()[0]
+        except IndexError:
+            return False # word is ''
+        
+        if firstword in PL_v_irregular_pres.keys():
+            return "%s%s" % (PL_v_irregular_pres[firstword],
+                                word[len(firstword):])
 
 # HANDLE IRREGULAR FUTURE, PRETERITE AND PERFECT TENSES 
 
-        mo = search(r"^(%s)((\s.*)?)$" % PL_v_irregular_non_pres,
-                    word, IGNORECASE)
-        if mo: return word
+        if firstword in PL_v_irregular_non_pres:
+            return word
 
 # HANDLE PRESENT NEGATIONS (SIMPLE AND COMPOUND)
 
-        mo = search(r"^(%s)(n't(\s.*)?)$" % PL_v_irregular_pres_keys,
-                    word, IGNORECASE)
-        if mo: return "%s%s" % (PL_v_irregular_pres[mo.group(1).lower()],
-                                 mo.group(2))
+        if firstword.endswith("n't") and firstword[:-3] in PL_v_irregular_pres.keys():
+            return "%sn't%s" % (PL_v_irregular_pres[firstword[:-3]],
+                                 word[len(firstword):])
 
-        mo = search(r"^\S+n't\b",
-                    word, IGNORECASE)
-        if mo: return word
+        if firstword.endswith("n't"):
+            return word
 
 # HANDLE SPECIAL CASES
 
         mo = search(r"^(%s)$" % PL_v_special_s, word)
-        if mo: return False
-        if search(r"\s", word): return False
+        if mo:
+            return False
+        if search(r"\s", word):
+            return False
 
 
 # HANDLE STANDARD 3RD PERSON (CHOP THE ...(e)s OFF SINGLE WORDS)
 
         mo = search(r"^(.*)([cs]h|[x]|zz|ss)es$",
                     word, IGNORECASE)
-        if mo: return "%s%s" % (mo.group(1), mo.group(2))
+        if mo:
+            return "%s%s" % (mo.group(1), mo.group(2))
 
         mo = search(r"^(..+)ies$",
                     word, IGNORECASE)
-        if mo: return "%sy" % mo.group(1)
+        if mo:
+            return "%sy" % mo.group(1)
 
-        mo = search(r"(%s)$" % PL_v_oes_oe,
-                    word, IGNORECASE)
-        if mo: return word[:-1]
+        if (lowerword in PL_v_oes_oe or
+            lowerword[-4:] in PL_v_oes_oe_endings_size4 or
+            lowerword[-5:] in PL_v_oes_oe_endings_size5):
+                return word[:-1]
 
-        mo = search(r"(.+)oes$",
-                    word, IGNORECASE)
-        if mo: return word[:-2]
+        if lowerword.endswith('oes') and len(lowerword)>3:
+            return lowerword[:-2]
 
         mo = search(r"^(.*[^s])s$",
                     word, IGNORECASE)
-        if mo: return mo.group(1)
+        if mo:
+            return mo.group(1)
 
 # OTHERWISE, A REGULAR VERB (HANDLE ELSEWHERE)
 
@@ -1413,20 +1476,23 @@ class engine:
     def _PL_general_verb(self, word, count=None):
         count = self.get_count(count)
 
-        if count==1: return word
+        if count==1:
+            return word
 
 # HANDLE AMBIGUOUS PRESENT TENSES  (SIMPLE AND COMPOUND)
 
         mo = search(r"^(%s)((\s.*)?)$" % PL_v_ambiguous_pres_keys,
                     word, IGNORECASE)
-        if mo: return "%s%s" % (PL_v_ambiguous_pres[mo.group(1).lower()],
+        if mo:
+            return "%s%s" % (PL_v_ambiguous_pres[mo.group(1).lower()],
                                  mo.group(2))
 
 # HANDLE AMBIGUOUS PRETERITE AND PERFECT TENSES 
 
         mo = search(r"^(%s)((\s.*)?)$" % PL_v_ambiguous_non_pres,
                     word, IGNORECASE)
-        if mo: return word
+        if mo:
+            return word
 
 # OTHERWISE, 1st OR 2ND PERSON IS UNINFLECTED
 
@@ -1436,24 +1502,28 @@ class engine:
     def _PL_special_adjective(self, word, count=None):
         count = self.get_count(count)
 
-        if count==1: return word
+        if count==1:
+            return word
 
 # HANDLE USER-DEFINED ADJECTIVES
 
         value = self.ud_match(word, self.PL_adj_user_defined)
-        if value is not None: return value
+        if value is not None:
+            return value
 
 # HANDLE KNOWN CASES
 
         mo = search(r"^(%s)$" % PL_adj_special_keys,
                     word, IGNORECASE)
-        if mo: return "%s" % (PL_adj_special[mo.group(1).lower()])
+        if mo:
+            return "%s" % (PL_adj_special[mo.group(1).lower()])
 
 # HANDLE POSSESSIVES
 
         mo = search(r"^(%s)$" % PL_adj_poss_keys,
                     word, IGNORECASE)
-        if mo: return "%s" % (PL_adj_poss[mo.group(1).lower()])
+        if mo:
+            return "%s" % (PL_adj_poss[mo.group(1).lower()])
 
         mo = search(r"^(.*)'s?$",
                     word)
@@ -1474,12 +1544,13 @@ class engine:
                     mystr, IGNORECASE)
         if mo:
             word = mo.group(2)
-            if not word: return mystr
+            if not word:
+                return mystr
             pre = mo.group(1)
             post = mo.group(3)
             result = self._indef_article(word, count)
             return "%s%s%s" % (pre, result, post)
-        return '' #TODO: is this what should be returned on no match?
+        return ''
 
     AN = A
 
@@ -1487,13 +1558,15 @@ class engine:
     def _indef_article(self, word, count):
         mycount = self.get_count(count)
 
-        if mycount!=1: return "%s %s" % (count, word)
+        if mycount!=1:
+            return "%s %s" % (count, word)
 
 
 # HANDLE USER-DEFINED VARIANTS
 
         value = self.ud_match(word, self.A_a_user_defined)
-        if value is not None: return "%s %s" % (value, word)
+        if value is not None:
+            return "%s %s" % (value, word)
 
 
 # HANDLE SPECIAL CASES
@@ -1504,7 +1577,8 @@ class engine:
                   (r"^[bcdgjkpqtuvwyz]$", "a"),
                  ):
             mo = search(a[0], word, IGNORECASE)
-            if mo: return "%s %s" % (a[1], word)
+            if mo:
+                return "%s %s" % (a[1], word)
 
 # HANDLE ABBREVIATIONS
 
@@ -1514,12 +1588,14 @@ class engine:
                   (r"^[a-z][.-]", "a", IGNORECASE),
                  ):
             mo = search(a[0], word, a[2])
-            if mo: return "%s %s" % (a[1], word)
+            if mo:
+                return "%s %s" % (a[1], word)
 
 # HANDLE CONSONANTS
 
         mo = search(r"^[^aeiouy]", word, IGNORECASE)
-        if mo: return "a %s" % word
+        if mo:
+            return "a %s" % word
 
 # HANDLE SPECIAL VOWEL-FORMS
 
@@ -1530,22 +1606,26 @@ class engine:
                   (r"^u[bcfhjkqrst][aeiou]", "a"),
                  ):
             mo = search(a[0], word, IGNORECASE)
-            if mo: return "%s %s" % (a[1], word)
+            if mo:
+                return "%s %s" % (a[1], word)
 
 # HANDLE SPECIAL CAPITALS
 
         mo = search(r"^U[NK][AIEO]?", word)
-        if mo: return "a %s" % word
+        if mo:
+            return "a %s" % word
 
 # HANDLE VOWELS
 
         mo = search(r"^[aeiou]", word, IGNORECASE)
-        if mo: return "an %s" % word
+        if mo:
+            return "an %s" % word
 
 # HANDLE y... (BEFORE CERTAIN CONSONANTS IMPLIES (UNNATURALIZED) "i.." SOUND)
 
         mo = search(r"^(%s)" % A_y_cons, word, IGNORECASE)
-        if mo: return "an %s" % word
+        if mo:
+            return "an %s" % word
 
 # OTHERWISE, GUESS "a"
         return "a %s" % word
@@ -1644,32 +1724,45 @@ class engine:
     
     def group1sub(self, mo):
         units = int(mo.group(1))
-        if units == 1: return " %s, " % self.number_args['one']
-        elif units: return "%s, " % unit[units] #TODO: bug one and zero are padded with a space but other numbers aren't. check this in perl
-        else: return " %s, " % self.number_args['zero']
+        if units == 1:
+            return " %s, " % self.number_args['one']
+        elif units:
+            return "%s, " % unit[units] #TODO: bug one and zero are padded with a space but other numbers aren't. check this in perl
+        else:
+            return " %s, " % self.number_args['zero']
     
     def group1bsub(self, mo):
         units = int(mo.group(1))
-        if units: return "%s, " % unit[units] #TODO: bug one and zero are padded with a space but other numbers aren't. check this in perl
-        else: return " %s, " % self.number_args['zero']
+        if units:
+            return "%s, " % unit[units] #TODO: bug one and zero are padded with a space but other numbers aren't. check this in perl
+        else:
+            return " %s, " % self.number_args['zero']
     
     def group2sub(self, mo):
         tens = int(mo.group(1))
         units = int(mo.group(2))
-        if tens: return "%s, " % self.tenfn(tens, units)
-        if units: return " %s %s, " % (self.number_args['zero'],unit[units])
+        if tens:
+            return "%s, " % self.tenfn(tens, units)
+        if units:
+            return " %s %s, " % (self.number_args['zero'],unit[units])
         return " %s %s, " % (self.number_args['zero'], self.number_args['zero'])
     
     def group3sub(self, mo):
         hundreds = int(mo.group(1))
         tens = int(mo.group(2))
         units = int(mo.group(3))
-        if hundreds==1: hunword = " %s" % self.number_args['one']
-        elif hundreds: hunword = "%s" % unit[units] #TODO: bug one and zero are padded with a space but other numbers aren't. check this in perl
-        else: hunword = " %s" % self.number_args['zero']
-        if tens: tenword = self.tenfn(tens, units)
-        elif units: tenword = " %s %s" % (self.number_args['zero'], unit[units])
-        else: tenword = " %s %s" % (self.number_args['zero'], self.number_args['zero'])
+        if hundreds==1:
+            hunword = " %s" % self.number_args['one']
+        elif hundreds:
+            hunword = "%s" % unit[units] #TODO: bug one and zero are padded with a space but other numbers aren't. check this in perl
+        else:
+            hunword = " %s" % self.number_args['zero']
+        if tens:
+            tenword = self.tenfn(tens, units)
+        elif units:
+            tenword = " %s %s" % (self.number_args['zero'], unit[units])
+        else:
+            tenword = " %s %s" % (self.number_args['zero'], self.number_args['zero'])
         return "%s %s, " % (hunword, tenword)
     
     def hundsub(self, mo):
@@ -1748,7 +1841,8 @@ class engine:
             spnum = num.split('.',1)
             while (self.number_args['comma']):
                     (spnum[0], n) = subn(r"(\d)(\d{3}(?:,|\Z))",r"\1,\2", spnum[0])
-                    if n==0: break
+                    if n==0:
+                        break
             try:
                 return "%s.%s" % (spnum[0], spnum[1])
             except IndexError:
@@ -1758,16 +1852,17 @@ class engine:
             raise BadChunkingOptionError 
         nowhite = num.lstrip()
         if nowhite[0] == '+':
-            sign = "+"
+            sign = "plus"
         elif nowhite[0] == '-':
-            sign = "-"
+            sign = "minus"
         else:
             sign = ""
         comma = self.number_args['comma']
         andword = self.number_args['andword'] #can't use 'and' keyword in **kwds
     
         myord =  (num[-2:] in ('st', 'nd', 'rd', 'th'))
-        if myord: num = num[:-2]
+        if myord:
+            num = num[:-2]
         if self.number_args['decimal']:
             if group != 0:
                 chunks = num.split('.')
@@ -1782,27 +1877,31 @@ class engine:
 
         if chunks[0] == '':
             first = 0
-            if len(chunks) > 1: loopstart = 1
+            if len(chunks) > 1:
+                loopstart = 1
 
-        for i in range(loopstart, len(chunks)): 
+        for i in range(loopstart, len(chunks)):
             chunk = chunks[i]
             #remove all non numeric \D
             chunk = resub(r"\D", self.blankfn, chunk)
-            if chunk == "": chunk = "0"
+            if chunk == "":
+                chunk = "0"
 
             if group != 0 and first != 0:
                 chunk = self.enword(chunk, 1)
             else:
                 chunk = self.enword(chunk, group)
 
-            if chunk[-2:] == ', ': chunk = chunk[:-2]
+            if chunk[-2:] == ', ':
+                chunk = chunk[:-2]
             chunk = resub(r"\s+,", self.commafn, chunk)
             if group != 0 and first:
                 chunk = resub(r", (\S+)\s+\Z", "%s \1" % andword, chunk)
             chunk = resub(r"\s+", self.spacefn, chunk)
             #chunk = resub(r"(\A\s|\s\Z)", self.blankfn, chunk)
             chunk = chunk.strip()
-            if first: first = ''
+            if first:
+                first = ''
             chunks[i] = chunk
 
         numchunks = []
@@ -1846,33 +1945,41 @@ class engine:
 
 # Join words with commas and a trailing 'and' (when appropriate)...
 
-    def WORDLIST(self, *words, **opt):
-        if not words: return ""
-        if len(words) == 1: return words[0] 
- 
-        conj = opt.get('conj', 'and')
-        if len(words) == 2:
-            conj = resub(r"^(?=[^\W\d_])|(?<=[^\W\d_])$", self.spacefn, conj)
-            return "%s%s%s" % (words[0], conj, words[1])
- 
-        try:
-            sep = opt['sep']
-        except KeyError:
-            if ',' in ''.join(words):
-                sep = '; '
-            else:
-                sep = ', '
-    
-        if 'final_sep' not in opt:
-            final_sep = "%s %s" % (sep, conj)
-        else:
-            if len(opt['final_sep']) == 0:
-                final_sep = conj
-            else:
-                final_sep = "%s %s" % (opt['final_sep'], conj)
+    def WORDLIST(self, words, sep=None, sep_spaced=True,
+                 final_sep=None,
+                 conj='and', conj_spaced=True):
+        '''
+        options:
+        conj: replacement for 'and'
+        sep: separator. default ',', unless ',' is in the list then ';'
+        final_sep: final separator. default ',', unless ',' is in the list then ';'
+        conj_spaced: boolean. Should conj have spaces around it
+        
 
-        final_sep = resub(r"\s+", self.spacefn ,final_sep)
-        final_sep = resub(r"^(?=[^\W\d_])|(?<=[^\W\d_])$", self.spacefn, final_sep)
+        '''
+        if not words:
+            return ""
+        if len(words) == 1:
+            return words[0] 
+
+        if conj_spaced:
+            conj = ' %s ' % conj
+
+        if len(words) == 2:
+            return "%s%s%s" % (words[0], conj, words[1])
+
+        if sep is None:
+            if ',' in ''.join(words):
+                sep = ';'
+            else:
+                sep = ','
+        if final_sep is None:
+            final_sep = sep
     
+        final_sep = "%s%s" % (final_sep, conj)
+
+        if sep_spaced:
+            sep += ' '
+            
         return "%s%s%s" %(sep.join(words[0:-1]), final_sep, words[-1])
 
