@@ -6,7 +6,8 @@ from re import error as reerror
 import inflect
 
 reload(inflect)
-from inflect import BadChunkingOptionError, NumOutOfRangeError
+from inflect import BadChunkingOptionError, NumOutOfRangeError, UnknownClassicalModeError
+from inflect import UnknownClassicalModeError, BadNumValueError
 
 class test(unittest.TestCase):
     def test_enclose(self):
@@ -65,7 +66,7 @@ class test(unittest.TestCase):
         mydict.update(dict(names=1,zero=0))
         self.assertEqual(p.classical_dict, mydict)
 
-
+        self.assertRaises(UnknownClassicalModeError, p.classical, 'bogus', reallybogus=1)
 
     def test_num(self):
         # def NUM
@@ -94,6 +95,7 @@ class test(unittest.TestCase):
         self.assertEqual(p.persistent_count, 3)
         self.assertEqual(ret, '')
 
+        self.assertRaises(BadNumValueError, p.NUM, 'text')
 
     def test_inflect(self):
         p = inflect.engine()
@@ -107,26 +109,18 @@ class test(unittest.TestCase):
                         ):
             self.assertEqual(p.inflect(txt), ans)
 
-        p.PLmo = lambda mo: "mockPL"
-        p.PL_Nmo = lambda mo: "mockPL_N"
-        p.PL_Vmo = lambda mo: "mockPL_V"
-        p.PL_ADJmo = lambda mo: "mockPL_ADJ"
-        p.Amo = lambda mo: "mockA"
-        p.NOmo = lambda mo: "mockNO"
-        p.ORDmo = lambda mo: "mockORD"
-        p.NUMWORDSmo = lambda mo: "mockNUMWORDS"
-        p.PART_PRESmo = lambda mo: "mockPART_PRES"
-
         for txt, ans in (
-        ("PL(rock)", "mockPL"),
-        ("PL(rock)  PL(child)", "mockPL  mockPL"),
-        ("NUM(2) PL(rock)  PL(child)", "2 mockPL  mockPL"),
+        ("PL(rock)", "rocks"),
+        ("PL(rock)  PL(child)", "rocks  children"),
+        ("NUM(2) PL(rock)  PL(child)", "2 rocks  children"),
 
         ("PL(rock) PL_N(rock) PL_V(rocks) PL_ADJ(big) A(ant)",
-                 "mockPL mockPL_N mockPL_V mockPL_ADJ mockA"),
+                 "rocks rocks rock big an ant"),
 
-        ("AN(rock) NO(0) ORD(3) NUMWORDS(1234) PART_PRES(runs)",
-                 "mockA mockNO mockORD mockNUMWORDS mockPART_PRES"),
+        ("AN(rock) NO(cat) ORD(3) NUMWORDS(1234) PART_PRES(runs)",
+                 "a rock no cats 3rd one thousand, two hundred and thirty-four running"),
+
+        ("A(cat,0) A(cat,1) A(cat,2) A(cat, 2)", "0 cat a cat 2 cat  2 cat"), # TODO: extra space when space before number. Is this desirable?
                         ):
             self.assertEqual(p.inflect(txt), ans)
 
@@ -173,10 +167,17 @@ class test(unittest.TestCase):
         self.assertEqual(p.ud_match('aviatrix',p.PL_sb_user_defined),
                         None)
 
+        p.def_noun('(cat)',r'$1s')
+        self.assertEqual(p.PL('cat'),'cats')
+
         inflect.STDOUT_ON = False
         self.assertRaises(inflect.BadUserDefinedPatternError, p.def_noun, '(??', None)
         inflect.STDOUT_ON = True
 
+        p.def_noun(None,'') # check None doesn't crash it
+
+
+        
         #def_verb
         p.def_verb('will','shall',
                          'will','will',
@@ -226,6 +227,7 @@ class test(unittest.TestCase):
                 ('cow   ',('', 'cow', '   ')),
                 ('  cow   ',('  ', 'cow', '   ')),
                 ('',('', '', '')),
+                ('bottle of beer',('', 'bottle of beer', '')),
                 #spaces give weird results
                 #(' '),('', ' ', '')),
                 #('  '),(' ', ' ', '')),
@@ -243,6 +245,7 @@ class test(unittest.TestCase):
                             (p.PL, 'knife', 'knives'),
                             (p.PL, 'knifes', 'knife'),
                             (p.PL, ' cat  ', ' cats  '),
+                            (p.PL, 'court martial', 'courts martial'),
                             (p.PL_N, '',''),
                             (p.PL_N, 'cow','cows'),
                             (p.PL_N, 'thought','thoughts'),
@@ -279,6 +282,7 @@ class test(unittest.TestCase):
                     (p.PL_eq, 'taxi', 'taxes', False),
                     (p.PL_eq, 'time', 'Times', False),
                     (p.PL_eq, 'time'.lower(), 'Times'.lower(), 's:p'),
+                    (p.PL_eq, 'courts martial', 'court martial', 'p:s'),
                     (p.PL_N_eq, 'index','index','eq'),
                     (p.PL_N_eq, 'index','indexes','s:p'),
                     (p.PL_N_eq, 'index','indices','s:p'),
@@ -326,6 +330,7 @@ class test(unittest.TestCase):
         self.assertEqual(p._PL_check_plurals_ADJ("indexes's", "indexes's"), False)
         self.assertEqual(p._PL_check_plurals_ADJ("dogmas's", "dogmata's"), True)
         self.assertEqual(p._PL_check_plurals_ADJ("dogmas'", "dogmata'"), True)
+        self.assertEqual(p._PL_check_plurals_ADJ("indexes'", "indices'"), True)
 
     def test_count(self):
         p = inflect.engine()
@@ -374,6 +379,9 @@ class test(unittest.TestCase):
                 ('mother-in-law', 'mothers-in-law'),
                 ('about me', 'about us'),
                 #('about ME', 'about US'),
+                ('to it', 'to them'),
+                ('from it', 'from them'),
+                ('with it', 'with them'),
                 ('I', 'we'),
                 ('you', 'you'),
                 #('YOU', 'YOU'),
@@ -498,6 +506,9 @@ class test(unittest.TestCase):
         self.assertEqual(p._PL_special_verb('dresses'), 'dress')
         self.assertEqual(p._PL_special_verb('flies'), 'fly')
         self.assertEqual(p._PL_special_verb('canoes'), 'canoe')
+        self.assertEqual(p._PL_special_verb('horseshoes'), 'horseshoe')
+        self.assertEqual(p._PL_special_verb('does'), 'do')
+        self.assertEqual(p._PL_special_verb('zzzoes'), 'zzzo') # what's a real word to test this case?
         self.assertEqual(p._PL_special_verb('runs'), 'run')
 
     def test__PL_general_verb(self):
@@ -558,6 +569,11 @@ class test(unittest.TestCase):
                 ('OPEC', 'an OPEC'),
                 ('FAQ', 'a FAQ'),
                 ('UNESCO', 'a UNESCO'),
+                ('a','an a'),
+                ('an','an an'),
+                ('an ant','an ant'),
+                ('a cat','a cat'),
+                ('',''),
                 
                     ):
             self.assertEqual(p.A(sing), plur)
@@ -573,6 +589,9 @@ class test(unittest.TestCase):
         self.assertEqual(p.NO('cat', count=3), '3 cats')
         self.assertEqual(p.NO('cat', count='three'), 'three cats')
         self.assertEqual(p.NO('mouse'), 'no mice')
+        p.NUM(3)
+        self.assertEqual(p.NO('cat'), '3 cats')
+        
         
     def test_PART_PRES(self):
         p = inflect.engine()
@@ -722,6 +741,8 @@ class test(unittest.TestCase):
             ('100000', 'one hundred thousand'),
             ('1000000', 'one million'),
             ('10000000', 'ten million'),
+            ('+10', 'plus ten'),
+            ('-10', 'minus ten'),
             ):
             self.assertEqual(NUMWORDS(n), word)
             
@@ -746,6 +767,8 @@ class test(unittest.TestCase):
                          ['one million',
                           'two hundred and thirty-four thousand',
                           'five hundred and sixty-seven'])
+        self.assertEqual(NUMWORDS('+10', wantlist=True),
+                         None) #TODO: fix this!
         self.assertEqual(NUMWORDS('1234', andword=''),
                          'one thousand, two hundred thirty-four')
         self.assertEqual(NUMWORDS('1234', andword='plus'),
@@ -757,9 +780,9 @@ class test(unittest.TestCase):
         self.assertEqual(NUMWORDS('123.456', group=1, decimal='mark', one='one'),
                          'one, two, three, mark, four, five, six')
         self.assertEqual(NUMWORDS('12345', group=3),
-                         'one, two, three, four, five')
+                         'one, two, three, four, five') #TODO: group being ignored?
         self.assertEqual(NUMWORDS('12345', group=2),
-                         'one, two, three, four, five')
+                         'one, two, three, four, five') #TODO: group being ignored?
         self.assertEqual(NUMWORDS('12345', group=1),
                          'one, two, three, four, five')
         self.assertEqual(NUMWORDS('1234th', group=0, andword='and'),
@@ -792,18 +815,53 @@ class test(unittest.TestCase):
     def test_wordlist(self):
         p = inflect.engine()
         WORDLIST = p.WORDLIST
-        self.assertEqual(WORDLIST('apple', 'banana'),
+        self.assertEqual(WORDLIST([]),
+                         '')
+        self.assertEqual(WORDLIST(('apple',)),
+                         'apple')
+        self.assertEqual(WORDLIST(('apple', 'banana')),
                          'apple and banana')
-        self.assertEqual(WORDLIST('apple', 'banana', 'carrot'),
+        self.assertEqual(WORDLIST(('apple', 'banana', 'carrot')),
                          'apple, banana, and carrot')
-        self.assertEqual(WORDLIST('apple', '1,000', 'carrot'),
+        self.assertEqual(WORDLIST(('apple', '1,000', 'carrot')),
                          'apple; 1,000; and carrot')
-        self.assertEqual(WORDLIST('apple', 'banana', 'carrot', final_sep=""),
+        self.assertEqual(WORDLIST(('apple', '1,000', 'carrot'), sep=','),
+                         'apple, 1,000, and carrot')
+        self.assertEqual(WORDLIST(('apple', 'banana', 'carrot'), final_sep=""),
                          'apple, banana and carrot')
-        self.assertEqual(WORDLIST('apple', 'banana', 'carrot', conj="or"),
+        self.assertEqual(WORDLIST(('apple', 'banana', 'carrot'), final_sep=";"),
+                         'apple, banana; and carrot')
+        self.assertEqual(WORDLIST(('apple', 'banana', 'carrot'), conj="or"),
                          'apple, banana, or carrot')
+        
+        self.assertEqual(WORDLIST(('apple', 'banana'), conj=" or "),
+                         'apple  or  banana')
+        self.assertEqual(WORDLIST(('apple', 'banana'), conj="&"),
+                         'apple & banana') # TODO: want spaces here. Done, report upstream
+        self.assertEqual(WORDLIST(('apple', 'banana'), conj="&", conj_spaced=False),
+                         'apple&banana')
+        self.assertEqual(WORDLIST(('apple', 'banana'), conj="& ", conj_spaced=False),
+                         'apple& banana')
 
 
+        self.assertEqual(WORDLIST(('apple', 'banana', 'carrot'), conj=" or "),
+                         'apple, banana,  or  carrot')
+        self.assertEqual(WORDLIST(('apple', 'banana', 'carrot'), conj="+"),
+                         'apple, banana, + carrot')
+        self.assertEqual(WORDLIST(('apple', 'banana', 'carrot'), conj="&"),
+                         'apple, banana, & carrot') # TODO: want space here. Done, report updtream
+        self.assertEqual(WORDLIST(('apple', 'banana', 'carrot'), conj="&", conj_spaced=False),
+                         'apple, banana,&carrot') # TODO: want space here. Done, report updtream
+        self.assertEqual(WORDLIST(('apple', 'banana', 'carrot'), conj=" &", conj_spaced=False),
+                         'apple, banana, &carrot') # TODO: want space here. Done, report updtream
+
+
+
+    def test_print(self):
+        inflect.STDOUT_ON = True
+        inflect.print3('') # make sure it doesn't crash
+        inflect.STDOUT_ON = False
+        
     def test_doc_examples(self):
         p = inflect.engine()
         self.assertEqual(p.PL_N('I'), 'we')
@@ -857,6 +915,8 @@ class test(unittest.TestCase):
             ):
             self.assertEqual(p.A('%s %s' % (p.NUMWORDS(1, one='a'),word)), txt)
 
+        p.NUM(2)
+
 #TODO: test .inflectrc file code        
 
 if __name__ == "__main__":
